IDI_ICON1               ICON    DISCARDABLE     "QuiteRSS.ico"

#ifndef Q_CC_BOR
# if defined(UNDER_CE) && UNDER_CE >= 400
#  include <winbase.h>
# else
#  include <winver.h>
# endif
#endif
#include "src/VersionNo.h"
VS_VERSION_INFO VERSIONINFO
 FILEVERSION FILEVER
 PRODUCTVERSION PRODUCTVER
 FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x4L
 FILETYPE 0x1L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "000004b0"
        BEGIN
            VALUE "CompanyName", "QuiteRSS Team\0"
            VALUE "Comments", "Application\0"
            VALUE "FileDescription", "QuiteRSS\0"
            VALUE "FileVersion", STRFILEVER_FULL
            VALUE "InternalName", "QuiteRSS\0"
	    VALUE "OriginalFilename", "QuiteRSS.exe\0"
            VALUE "LegalCopyright", "Copyright © 2011-2017 QuiteRSS Team\0"
            VALUE "ProductName", "QuiteRSS\0"
            VALUE "ProductVersion", STRPRODUCTVER
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x0, 1200
    END
END
