/* ============================================================
* QuiteRSS is a open-source cross-platform RSS/Atom news feeds reader
* Copyright (C) 2011-2018 QuiteRSS Team <quiterssteam@gmail.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
* ============================================================ */
#ifndef SSLERRORDIALOG_H
#define SSLERRORDIALOG_H

#include "dialog.h"

#include <QSslCertificate>

class SslErrorDialog : public Dialog
{
  Q_OBJECT

public:
  enum Result { Yes, No, OnlyForThisSession };

  explicit SslErrorDialog(QWidget* parent = 0);
  ~SslErrorDialog();

  void setText(const QString &text);
  Result result();

  static QString certificateItemText(const QSslCertificate &cert);
  static QString clearCertSpecialSymbols(const QString &string);
  static QString clearCertSpecialSymbols(const QStringList &stringList);

private slots:
  void buttonClicked(QAbstractButton* button);

private:
  QLabel *errorLabel_;
  Result result_;

};

#endif // SSLERRORDIALOG_H
